package com.foreverwin.mesnac.common.ftp;

import com.foreverwin.mesnac.common.constant.IntegrationTypeConstant;
import com.foreverwin.mesnac.common.model.IntegrationLog;
import com.foreverwin.mesnac.common.service.IntegrationLogService;
import com.foreverwin.modular.core.exception.BaseException;
import com.jcraft.jsch.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;


/**
 * @Author Yinq
 * @create 2022-07-05 14:27
 */
@Component
public class JSchClient {
    private static final Logger logger = LoggerFactory.getLogger(JSchClient.class);
    private JSch jSch;
    private String ipAddress;   //主机ip
    private String username;   // 账号
    private String password;   // 密码
    private int port;  // 端口号
    // session对象
    private Session session;

    // JAVA与主机的连接通道
    private Channel channel;
    // sftp通道
    ChannelSftp chSftp;

    @Autowired
    private IntegrationLogService integrationLogService;

    public JSchClient() {
    }

    public JSchClient(String ipAddress, String username, String password, int port) {
        this.ipAddress = ipAddress;
        this.username = username;
        this.password = password;
        this.port = port;
    }

    /**
     * 通过SFTP连接服务器
     */
    public void connectSFTP() {
        try {
            JSch jsch = new JSch();
            jsch.getSession(username, ipAddress, port);
            session = jsch.getSession(username, ipAddress, port);

            if (logger.isInfoEnabled()) {
                logger.info("Session created.");
                logger.error("连接信息"+username+"/"+ipAddress+"/"+port+"/"+password);
            }
            session.setPassword(password);
            Properties sshConfig = new Properties();
            sshConfig.put("StrictHostKeyChecking", "no");
            session.setConfig(sshConfig);
            session.connect();
            if (logger.isInfoEnabled()) {
                logger.info("Session connected.");
            }
            Channel channel = session.openChannel("sftp");
            channel.connect();
            if (logger.isInfoEnabled()) {
                logger.info("Opening Channel.");
            }
            chSftp = (ChannelSftp) channel;
            if (logger.isInfoEnabled()) {
                logger.info("Connected to " + ipAddress + ".");
            }
        } catch (Exception e) {
            logger.error("图纸SFTP连接失败"+e.getMessage());
            logger.error(session.getUserName()+session.getHost()+session.getPort());
            throw new BaseException("错误一"+e.getMessage()+"连接信息"+username+"/"+ipAddress+"/"+port+"/"+password);
        }
    }

    /**
     * 获取指定文件流
     *
     * @param path
     * @return
     * @throws Exception
     */
    public InputStream getSFTP(String path) throws Exception {
        InputStream in = null;
        Channel channel = null;
        channel = session.openChannel("sftp");
        channel.connect();
        chSftp = (ChannelSftp) channel;
        in = chSftp.get(path);
        return in;
    }



    public InputStream downloadStream(String directory, String downloadFile){
        ChannelSftp sftp;
        try{
            sftp = (ChannelSftp) channel;
            if(directory != null && !"".equals(directory)){
                sftp.cd(directory);
            }
            return sftp.get(downloadFile);
        }catch (SftpException e){
            logger.error("文件下载异常！", e);
        }
        return null;
    }

    /**
     * 关闭连接
     */
    public void close() {
        if (channel != null && channel.isConnected()) {
            channel.disconnect();
        }
        if (session != null && session.isConnected()) {
            session.disconnect();
        }
        if (chSftp != null && chSftp.isConnected()) {
            chSftp.quit();
        }
    }

    /**
     * 文件上传至主机
     *
     * @param directory  当前文件路径
     * @param uploadFile 上传至主机的路径
     */
    public void upload(String directory, String uploadFile) {
        // 创建JSch对象
        jSch = new JSch();

        try {
            // 根据主机账号、ip、端口获取一个Session对象
            session = jSch.getSession(username, ipAddress, port);
            // 存放主机密码
            session.setPassword(password);
            // 去掉首次连接确认
            Properties config = new Properties();
            config.put("StrictHostKeyChecking", "no");
            session.setConfig(config);
            // 超时连接时间为3秒
            session.setTimeout(3000);
            // 进行连接
            session.connect();
            // 打开SFTP通道
            chSftp = (ChannelSftp) session.openChannel("sftp");
            // 建立STFP连接
            chSftp.connect();
            // 设置编码格式
            chSftp.setFilenameEncoding("UTF-8");
            /**
             * 说明：
             * 1、当前文件上传信息没有任何反馈，如果没有异常则代表成功
             * 2、如果需要判断是否读取成功的进度，可参考https://blog.csdn.net/coding99/article/details/52416373?locationNum=13&fps=1
             * 3、将src文件上传到dst路径中
             */
            chSftp.put(directory, uploadFile);
            logger.info("文件上传成功");
        } catch (JSchException | SftpException e) {
            logger.warn(e.getMessage());
        } finally {
            close();
        }
    }

    /**
     * 将主机文件下载至本地
     *
     * @param directory    下载到本地的位置
     * @param downloadFile 下载文件在虚拟机的位置
     */
    public void download(String directory, String downloadFile) {

        try {
            jSch = new JSch();
            // 根据主机账号、ip、端口获取一个Session对象
            session = jSch.getSession(username, ipAddress, port);
            // 存放主机密码
            session.setPassword(password);
            // 去掉首次连接确认
            Properties config = new Properties();
            config.put("StrictHostKeyChecking", "no");
            session.setConfig(config);
            // 超时连接时间为3秒
            session.setTimeout(3000);
            // 进行连接
            session.connect();
            // 打开SFTP通道
            chSftp = (ChannelSftp) session.openChannel("sftp");
            // 建立SFTP通道的连接
            chSftp.connect();
            // 设置编码格式
            chSftp.setFilenameEncoding("UTF-8");
            /**
             * 说明：
             * 1、当前上读取文件信息没有任何反馈，如果没有异常则代表成功
             * 2、如果需要判断是否读取成功的进度，可参考https://blog.csdn.net/coding99/article/details/52416373?locationNum=13&fps=1
             * 3、将src文件下载到dst路径中
             */
            chSftp.get(directory, downloadFile);
            logger.info("文件下载成功");
        } catch (JSchException | SftpException e) {
            logger.warn(e.getMessage());
        } finally {
            close();
        }
    }

    /**
     * 判断目录是否存在
     *
     * @param directory
     * @return
     */
    public boolean isDirExist(String directory) {
        boolean isDirExistFlag = false;
        try {
            SftpATTRS sftpATTRS = chSftp.lstat(directory);
            isDirExistFlag = true;
            return sftpATTRS.isDir();
        } catch (Exception e) {
            if (e.getMessage().toLowerCase().equals("no such file")) {
                isDirExistFlag = false;
            }
        }
        return isDirExistFlag;
    }


    public boolean isExistSftp(String filePach){
        boolean result = false;
        Session session = null;
        Channel channel = null;
        ChannelSftp sftp = null;
        try {
            JSch jsch = new JSch();
            session = jsch.getSession("root","172.16.77.182");
            session.setPassword("Ys0428##");
            session.setTimeout(60000);
            Properties config = new Properties();
            config.put("StrictHostKeyChecking", "no");
            session.setConfig(config);
            session.connect();

            channel = session.openChannel("sftp");
            channel.connect();
            sftp = (ChannelSftp) channel;
            SftpATTRS lstat = sftp.lstat(filePach);
            result = true;
        } catch (JSchException e) {
            e.printStackTrace();

        } catch (SftpException e) {
            e.printStackTrace();
        } catch (Exception e){
            e.printStackTrace();
        }finally {
            sftp.quit();
            channel.disconnect();
            session.disconnect();
        }
        return result;
    }





    /**
     * 列出目录下的文件
     *
     * @param directory：要列出的目录
     * @return
     * @throws SftpException
     */
    public List<String> listFiles(String directory) throws SftpException {

        List fileNameList = new ArrayList();

        Vector fileList = chSftp.ls(directory);
        Iterator it = fileList.iterator();

        while (it.hasNext()) {
            String fileName = ((ChannelSftp.LsEntry) it.next()).getFilename();
            if ((".".equals(fileName)) || ("..".equals(fileName))) {
                continue;
            }
            fileNameList.add(fileName);
        }
        return fileNameList;
    }

    /**
     * 下载文件
     *
     * @param directory    下载目录
     * @param downloadFile 下载的文件名
     * @return 字节数组
     */
    public ByteArrayInputStream getByteArrayInputStream(String directory, String downloadFile)
            throws SftpException, IOException {
        if (directory != null && !"".equals(directory)) {
            chSftp.cd(directory);
        }
        InputStream is = chSftp.get(downloadFile);
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        try {
            byte[] buffer = new byte[1024];
            int num = is.read(buffer);
            while (num != -1) {
                output.write(buffer, 0, num);
                num = is.read(buffer);
            }
            output.flush();
        } finally {
            if (is != null) {
                is.close();
            }
        }
        ByteArrayInputStream in = new ByteArrayInputStream(output.toByteArray());
        return in;
    }


}

